﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.Text;
using ERR_AudioPlayer;

namespace ERR_AudioPlayerWS
{

    [ServiceBehavior(InstanceContextMode = InstanceContextMode.Single)]
    public class AudioPlayerWS : IAudioPlayerWS
    {
        private AudioPlayer player = null; //Player used for playback
        private static string baseDir = Settings.Default.WavFileBaseDirectory; //Media files root direcotry
        private static TimeSpan tcFailover = new TimeSpan(0,0,0,0,0); //0 timecode for easier GUI frontend when there is no player initialized

        private static AudioLevel alFailover = new AudioLevel() { left = 0, right = 0 }; //0 audio level for easier GUI frontend when there is no player initialized

        /// <summary>
        /// DirectSound audio devices
        /// </summary>
        /// <returns>All usable DirectSound audio devices</returns>
        public List<SoundDevice> getDevices()
        {
            return AudioPlayer.getDevices();
        }

        /// <summary>
        /// Starts playback with new AudioPlayer instance while releasing resources from ongoing playback
        /// </summary>
        /// <param name="filename"></param>
        /// <param name="device"></param>
        /// <param name="tcIn"></param>
        public bool play(string filename, Guid device, TimeSpan tcIn)
        {
            if (player != null && player.IsPlaying) player.Stop();

            player = new AudioPlayer(baseDir + filename,tcIn, device, 50, true);

            return true;
        }

        /// <summary>
        /// Playback timespan from the beginning of the file to the current position being played
        /// </summary>
        public TimeSpan getTime()
        {
            if (player != null) return player.CurrentPosition;
            else return tcFailover;
        }

        /// <summary>
        /// Stops playback and releases the player for GC
        /// </summary>
        /// <returns>Always True</returns>
        public bool stop()
        {
            if (player != null)
            {
                player.Stop();
                player = null;
            }

            return true;
        }

        /// <summary>
        /// Returns current playback's audio levels
        /// </summary>
        public AudioLevel getLevels()
        {
            if (player != null) return player.level;
            else return alFailover;
        }

        /// <param name="hours"></param>
        /// <param name="minutes"></param>
        /// <param name="seconds"></param>
        /// <param name="milliseconds"></param>
        /// <returns>True: Operation Successful, False: Unable to seek</returns>
        public bool seekForward(int hours, int minutes, int seconds, int milliseconds)
        {
            if (player != null) return player.SeekForward(hours, minutes, seconds, milliseconds);

            return false;
        }

        /// <param name="hours"></param>
        /// <param name="minutes"></param>
        /// <param name="seconds"></param>
        /// <param name="milliseconds"></param>
        /// <returns>True: Operation Successful, False: Unable to seek</returns>
        public bool seekBackward(int hours, int minutes, int seconds, int milliseconds)
        {
            if (player != null) return player.SeekBackward(hours, minutes, seconds, milliseconds);

            return false;
        }

        // <returns>Last used device GUID or null when no device has been used yet</returns>
        public Guid? getDevice()
        {
            if (player != null) return player.GetDevice();
            else return null;
        }

    }
}
