﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Linq;
using System.ServiceModel;
using System.ServiceModel.Description;
using System.ServiceProcess;
using System.Text;
using System.Threading.Tasks;
using System.Net.Sockets;
using System.Net;
using System.Threading;
using System.IO;
using System.Xml.Serialization;

namespace ERR_AudioPlayerWS
{
    public partial class AudioPlayerWSHost : ServiceBase
    {
        private static Uri baseAddress = new Uri(Settings.Default.ServiceBaseAddress); //WCF Service endpoint address //TODO move to App.config
        private static ServiceHost host = new ServiceHost(typeof(ERR_AudioPlayerWS.AudioPlayerWS), baseAddress); //WCF Service host
        private static NetTcpBinding binding = new NetTcpBinding(); //WCF Endpoint type

        private static Thread listenerThread; //Used for TCP Socket
        private static bool tcpListenerEnabled = true; //Enable/Disable TCP Socket

        static void Main()
        {
            System.IO.Directory.SetCurrentDirectory(AppDomain.CurrentDomain.BaseDirectory); //Useful when writing log files to the service's install directory.

            try
            {
                //WCF Metadata endpoint configuration //TODO move to App.config
                ServiceMetadataBehavior smb = new ServiceMetadataBehavior();
                smb.HttpGetEnabled = true;
                smb.HttpGetUrl = new Uri(Settings.Default.ServiceBaseMetadataAddress);
                smb.MetadataExporter.PolicyVersion = PolicyVersion.Policy15;
                host.Description.Behaviors.Add(smb);

                //WCF Security
                binding.Security.Mode = SecurityMode.Message;
                binding.Security.Message.ClientCredentialType = MessageCredentialType.Windows;

                //WCF Client endpoint
                host.AddServiceEndpoint(typeof(IAudioPlayerWS), binding, "netTCPBinding", baseAddress);


                //Debugging mode if you need to run the Windows Service in Console Application mode from Visual Studio
                bool DEBUGGING = Settings.Default.Debugging;

                if (Debugger.IsAttached || DEBUGGING)
                {
                    AudioPlayerWSHost service = new AudioPlayerWSHost();
                    service.OnStart(new string[] { });
                    Console.Read();
                    service.OnStop();
                }
                else
                {
                    ServiceBase[] ServicesToRun;
                    ServicesToRun = new ServiceBase[] { new AudioPlayerWSHost() };

                    ServiceBase.Run(ServicesToRun);
                }


            }
            catch (Exception ex)
            {
                //Redirect to your preferred logger
                //log.Write(ex.ToString());
            }
        }

        public AudioPlayerWSHost()
        {
            InitializeComponent();
        }


        /// <summary>
        /// This method is being called when the Windows Service is Starting.
        /// Make all needed modifications and checks here if you need some magic to happen before opening WCF service host
        /// </summary>
        /// <param name="args"></param>
        protected override void OnStart(string[] args)
        {
            //Redirect to your preferred logger
            //log.Write("INIT");
            try
            {
                host.Open(); //Open WCF Service host

                //Redirect to your preferred logger
                //log.Write("Service hosted @ " + Settings.Default.ServiceBaseAddress + "  Service Description @ " + Settings.Default.ServiceBaseMetadataAddress);

                //Redirect to your preferred logger
                //log.Write("STARTING LISTENER THREAD");
                listenerThread = new Thread(new ThreadStart(tcpListen)); //TCP Socket thread
                listenerThread.Start();
            }
            catch (Exception ex)
            {
                //Redirect to your preferred logger
                //log.Write(ex.ToString());
            }
        }

        protected override void OnStop()
        {
            //Redirect to your preferred logger
            //log.Write("STOPPING");

            tcpListenerEnabled = false;

            System.Threading.Thread.Sleep(100); //TODO Use signaling


            try
            {
                host.Close();
            }
            catch (Exception ex)
            {
                //Redirect to your preferred logger
                //log.Write(ex.ToString());
            }
        }

        //The following TCP listener is based on Inscriber RTX Chyron protocol
        //You need to modify this for your own purposes or use WCF endpoint instead.
        //When using WCF endpoint you can remove all the code regarding TCP socket!
        private static ChyronDataPacket cachedPacket = new ChyronDataPacket();

        private static string cachedFilename = string.Empty;
        private static string cachedAudioDevice = string.Empty;
        private static TimeSpan cachedInPoint = new TimeSpan(0,0,0,1,0);

        private static byte[] start = Encoding.GetEncoding(1200).GetBytes(">>!START!<<");
        private static byte[] stop = Encoding.GetEncoding(1200).GetBytes(">>!STOP!<<");

        private static void tcpListen()
        {

            //Redirect to your preferred logger
            //log.Write("TCP LISTENING @ " + Settings.Default.tcpHostIp +":" + Settings.Default.tcpHostPort);
            IPAddress ipDest = IPAddress.Parse(Settings.Default.tcpHostIp);
            IPEndPoint endpoint = new IPEndPoint(IPAddress.Any, Settings.Default.tcpHostPort);
            TcpListener listener = new TcpListener(endpoint);
            listener.Start();

            while (tcpListenerEnabled)
            {
                try
                {
                    //Redirect to your preferred logger
                    //log.Write("TCP WAITING FOR CONNECTION");
                    using (TcpClient client = listener.AcceptTcpClient())
                    {
                        //Redirect to your preferred logger
                        //log.Write("TCP CONNECTED");

                        NetworkStream stream = client.GetStream();

                        using (BinaryWriter w = new BinaryWriter(stream))
                        {
                            using (BinaryReader r = new BinaryReader(stream))
                            {
                                while (tcpListenerEnabled)
                                {

                                    byte[] data = new byte[client.ReceiveBufferSize];
                                    int bytesRead = 0;

                                    bytesRead = r.Read(data, 0, client.ReceiveBufferSize);

                                    if (bytesRead != 0)
                                    {
                                        //Direct channel to our hosted WCF Service to accept commands from TCP Socket with Chyron protocol
                                        ChannelFactory<IAudioPlayerWS> wcfClient = new ChannelFactory<IAudioPlayerWS>(binding, new EndpointAddress(Settings.Default.ServiceBaseAddress + "netTCPBinding"));
                                        IAudioPlayerWS wcf = wcfClient.CreateChannel();

                                        try
                                        {
                                            if (data.Take(start.Length).SequenceEqual(start)) //Take because our data buffer has trailing 00000
                                            {

                                                //Redirect to your preferred logger
                                                //log.Write("START " + cachedFilename + "  " + cachedAudioDevice + "\n");
                                                w.Write(wcf.play(cachedFilename, new Guid(cachedAudioDevice), cachedInPoint)); //PLAY
                                            }
                                            else if (data.Take(stop.Length).SequenceEqual(stop)) //Take because our data buffer has trailing 00000
                                            {
                                                //Redirect to your preferred logger
                                                //log.Write("STOP" + "\n");
                                                w.Write(wcf.stop()); //STOP
                                            }
                                            else
                                            {
                                                try
                                                {
                                                    XmlSerializer serializer = new XmlSerializer(typeof(ChyronDataPacket));
                                                    TextReader textReader = new StringReader(Encoding.GetEncoding(1200).GetString(data));

                                                    cachedPacket = (ChyronDataPacket)serializer.Deserialize(textReader);

                                                    textReader.Close();

                                                    cachedFilename = cachedPacket.TagName1;
                                                    
                                                    //We send Guard audio device GUID in TagText2.
                                                    if (!Settings.Default.isGuardService) cachedAudioDevice = cachedPacket.TagText1;
                                                    else cachedAudioDevice = cachedPacket.TagText2;

                                                    if (!TimeSpan.TryParseExact(cachedPacket.TagName2, "hh':'mm':'ss':'FFF", System.Globalization.CultureInfo.InvariantCulture, out cachedInPoint)) cachedInPoint = new TimeSpan(0, 0, 0, 1, 0);
                                                    //Redirect to your preferred logger
                                                    //else log.Write("ERROR: Failed to parse TagName2(InPoint)");

                                                    //Redirect to your preferred logger
                                                    //log.Write("TagName1(Filename):" + cachedPacket.TagName1 + "\n" +
                                                    //    "TagText1(Audio device GUID):" + cachedPacket.TagText1 + "\n" +
                                                    //    "TagName2(InPoint):" + cachedPacket.TagName2 + "\n" +
                                                    //    "TagText2(Audio device GUID(GUARD)):" + cachedPacket.TagText2);

                                                    //If file exists we are OK to go.
                                                    w.Write(new FileInfo(Settings.Default.WavFileBaseDirectory + cachedFilename).Exists);
                                                }
                                                catch (Exception ex)
                                                {
                                                    //Redirect to your preferred logger
                                                    //log.Write("PARSER FAILURE: RECEIVED DATA: " + Encoding.GetEncoding(1200).GetString(data) + "\n");
                                                    //Redirect to your preferred logger
                                                    //log.Write("ERR: MSG: " + ex.ToString() + "\n");
                                                    w.Write(false);
                                                }
                                            }

                                            wcfClient.Close();
                                        }
                                        catch (Exception ex)
                                        {
                                            //Redirect to your preferred logger
                                            //log.Write("Service inner communication failure: " + ex.ToString());
                                            wcfClient.Abort();
                                        }
                                    }
                                    else
                                    {
                                        w.Write(false);
                                        break;
                                    }
                                }
                            }
                        }
                    }
                }
                catch (Exception ex)
                {
                    //Redirect to your preferred logger
                    //log.Write(ex.ToString());
                }
            }
        }
    }
}
